==> OpenCL-0.7.7.tgz: OpenCL-0.7.7/src/filter.cpp <==
/*************************************************
* Filter Source File                             *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#include <opencl/filter.h>
#include <opencl/secqueue.h>

namespace OpenCL {

/*************************************************
* Filter Constructor                             *
*************************************************/
Filter::Filter(u32bit count) : port_count(count)
   {
   if(port_count) next = new Filter*[port_count];
   else           next = 0;
   for(u32bit j = 0; j != port_count; j++)
      next[j] = 0;
   port_num = 0;
   }

/*************************************************
* Send data to all ports                         *
*************************************************/
void Filter::send(const byte input[], u32bit length)
   {
   bool nothing_attached = true;
   for(u32bit j = 0; j != total_ports(); j++)
      if(next[j])
         {
         if(write_queue.size())
            next[j]->write(write_queue, write_queue.size());
         next[j]->write(input, length);
         nothing_attached = false;
         }
   if(nothing_attached)
      write_queue.append(input, length);
   else if(write_queue.has_items())
      write_queue.destroy();
   }

/*************************************************
* Close a Filter                                 *
*************************************************/
void Filter::close()
   {
   final();
   for(u32bit j = 0; j != total_ports(); j++)
      if(next[j])
         next[j]->close();
   }

/*************************************************
* Attach a filter to the current port            *
*************************************************/
void Filter::attach(Filter* new_filter)
   {
   if(new_filter)
      {
      Filter* next_to_last = this, *last = this;
      while(last->get_next())
         {
         next_to_last = last;
         last = last->get_next();
         }
      if(dynamic_cast<SecureQueue*>(last))
         {
         SecureQueue queue = *static_cast<SecureQueue*>(last);
         delete last;
         add(queue, new_filter);
         next_to_last->next[next_to_last->current_port()] = new_filter;
         }
      else
         last->next[last->current_port()] = new_filter;
      }
   }

/*************************************************
* Attach the queue onto each leaf of filter      *
*************************************************/
void Filter::add(const SecureQueue& queue, Filter* filter)
   {
   for(u32bit j = 0; j != filter->total_ports(); j++)
      if(!filter->next[j])
         filter->next[j] = new SecureQueue(queue);
      else
         add(queue, filter->next[j]);
   }

/*************************************************
* Set the active port on a filter                *
*************************************************/
void Filter::set_port(u32bit new_port) throw(Invalid_Port_Number)
   {
   if(new_port >= total_ports())
      throw Invalid_Port_Number();
   port_num = new_port;
   }

/*************************************************
* Destroy this Filter and it's children          *
*************************************************/
void destroy(Filter*& to_kill)
   {
   if(!to_kill) return;
   for(u32bit j = 0; j != to_kill->total_ports(); j++)
      destroy(to_kill->next[j]);
   delete to_kill;
   to_kill = 0;
   }

}

==> OpenCL-0.7.8.tgz: OpenCL-0.7.8/src/filter.cpp <==
/*************************************************
* Filter Source File                             *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/filter.h>
#include <opencl/secqueue.h>

namespace OpenCL {

/*************************************************
* Filter Constructor                             *
*************************************************/
Filter::Filter(u32bit count)
   {
   next = 0;
   set_port_count(count);
   }

/*************************************************
* Set/Reset next                                 *
*************************************************/
void Filter::set_port_count(u32bit n)
   {
   port_count = n;
   if(next) delete[] next;
   if(port_count) next = new Filter*[port_count];
   else           next = 0;
   for(u32bit j = 0; j != port_count; j++)
      next[j] = 0;
   port_num = 0;
   filter_owns = 0;
   }

/*************************************************
* Send data to all ports                         *
*************************************************/
void Filter::send(const byte input[], u32bit length)
   {
   bool nothing_attached = true;
   for(u32bit j = 0; j != total_ports(); j++)
      if(next[j])
         {
         if(write_queue.size())
            next[j]->write(write_queue, write_queue.size());
         next[j]->write(input, length);
         nothing_attached = false;
         }
   if(nothing_attached)
      write_queue.append(input, length);
   else if(write_queue.has_items())
      write_queue.destroy();
   }

/*************************************************
* Start a new message                            *
*************************************************/
void Filter::new_msg()
   {
   start_msg();
   for(u32bit j = 0; j != total_ports(); j++)
      if(next[j])
         next[j]->new_msg();
   }

/*************************************************
* End the current message                        *
*************************************************/
void Filter::finish_msg()
   {
   end_msg();
   for(u32bit j = 0; j != total_ports(); j++)
      if(next[j])
         next[j]->finish_msg();
   }

/*************************************************
* Attach a filter to the current port            *
*************************************************/
void Filter::attach(Filter* new_filter)
   {
   if(new_filter)
      {
      Filter* next_to_last = this, *last = this;
      while(last->get_next())
         {
         next_to_last = last;
         last = last->get_next();
         }
      if(dynamic_cast<SecureQueue*>(last))
         {
         throw Exception("FIXME (attach): This should never happen (*please* report!)");
         SecureQueue queue = *static_cast<SecureQueue*>(last);
         delete last;
         add(queue, new_filter);
         next_to_last->next[next_to_last->current_port()] = new_filter;
         }
      else
         last->next[last->current_port()] = new_filter;
      }
   }

/*************************************************
* Attach the queue onto each leaf of filter      *
*************************************************/
void Filter::add(const SecureQueue& queue, Filter* filter)
   {
   for(u32bit j = 0; j != filter->total_ports(); j++)
      if(!filter->next[j])
         filter->next[j] = new SecureQueue(queue);
      else
         add(queue, filter->next[j]);
   }

/*************************************************
* Set the active port on a filter                *
*************************************************/
void Filter::set_port(u32bit new_port)
   {
   if(new_port >= total_ports())
      throw Invalid_Argument("Filter: Invalid port number");
   port_num = new_port;
   }

/*************************************************
* Destroy this Filter and it's children          *
*************************************************/
void destroy(Filter*& to_kill)
   {
   if(!to_kill) return;
   for(u32bit j = 0; j != to_kill->total_ports(); j++)
      destroy(to_kill->next[j]);
   delete to_kill;
   to_kill = 0;
   }

}
